#include <imsls.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

void l_print_LSD(int n1, int* equalMeans, float *means);

int main()
{
   char **anova_row_labels = NULL;
   char *col_labels[] = {" ", "\nID", "\nDF", "\nSSQ  ", 
      "Mean  \nsquares", "\nF-Test", "\np-Value"};
   float alpha = 0.05;
   int i, l, page_width = 132;  
   int n            = 80; /* Total number of observations      */
   int n_locations  = 1;  /* Number of locations               */
   int n_treatments =16;  /* Number of treatments              */
   int n_reps       = 5;  /* Number of replicates              */
   int n_blocks     =20;  /* Total number of blocks            */
   int n_aov_rows   = 7;  /* Number of rows in the anova table */

   int rep[]={                        
      1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
         2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 
         3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
         4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
         5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5
   };

   int block[]={                        
      1,  1,  1,  1,  2,  2,  2,  2,  3,  3,  3,  3,  4,  4,  4,  4,
      5,  5,  5,  5,  6,  6,  6,  6,  7,  7,  7,  7,  8,  8,  8,  8, 
      9,  9,  9,  9, 10, 10, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12,
      13, 13, 13, 13, 14, 14, 14, 14, 15, 15, 15, 15, 16, 16, 16, 16,
      17, 17, 17, 17, 18, 18, 18, 18, 19, 19, 19, 19, 20, 20, 20, 20
   };

   int treatment[]={        
      1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 
      1,  5,  9, 13, 10,  2, 14,  6,  7, 15,  3, 11, 16,  8, 12,  4,  
      1,  6, 11, 16,  5,  2, 15, 12,  9, 14,  3,  8, 13, 10,  7,  4,  
      1, 14,  7, 12, 13,  2, 11,  8,  5, 10,  3, 16,  9,  6, 15,  4,  
      1, 10, 15,  8,  9,  2,  7, 16, 13,  6,  3, 12,  5, 14, 11,  4
   };

   float y[] ={
      147, 152, 167, 150, 127, 155, 162, 172, 
         147, 100, 192, 177, 155, 195, 192, 205,
         140, 165, 182, 152,  97, 155, 192, 142,
         155, 182, 192, 192, 182, 207, 232, 162,
         155, 132, 177, 152, 182, 130, 177, 165,
         137, 185, 152, 152, 185, 122, 182, 192,
         220, 202, 175, 205, 205, 152, 180, 187,
         165, 150, 200, 160, 155, 177, 185, 172,
         147, 112, 177, 147, 180, 205, 190, 167,
         172, 212, 197, 192, 177, 220, 205, 225
   };

   float grand_mean;
   float cv;
   float *aov;
   float *treatment_means;
   float *std_err;
   int   *equal_means;
   int   df;

   aov = imsls_f_lattice(n, n_locations, n_reps, n_blocks,  
      n_treatments, rep, block, treatment, y, 
      IMSLS_GRAND_MEAN, &grand_mean, 
      IMSLS_CV, &cv,
      IMSLS_TREATMENT_MEANS, &treatment_means, 
      IMSLS_STD_ERRORS, &std_err,
      IMSLS_ANOVA_ROW_LABELS, &anova_row_labels,
      0);

   imsls_page(IMSLS_SET_PAGE_WIDTH, &page_width);
   /* Print the ANOVA table. */
   imsls_f_write_matrix("   *** ANALYSIS OF VARIANCE TABLE ***", 
      7, 6, aov, 
      IMSLS_WRITE_FORMAT, "%3.0f%3.0f%8.2f%7.2f%7.2f%7.3f",
      IMSLS_ROW_LABELS, anova_row_labels,
      IMSLS_COL_LABELS, col_labels,
      0);

   printf("\n\nAdjusted Grand Mean:      %7.3f", grand_mean);
   printf("\n\nCoefficient of Variation: %7.3f\n\n", cv);
   l = 0;
   printf("Adjusted Treatment Means: \n");
   for (i=0; i < n_treatments; i++){
      printf("treatment[%2d]             %7.4f \n", i+1, 
         treatment_means[l++]);
   }
   df = (int)std_err[3];
   printf("\nStandard Error for Comparing Two Adjusted Treatment ");
   printf("Means: %f \n(df=%d)\n", std_err[2], df);
   equal_means = imsls_f_multiple_comparisons(n_treatments, 
      treatment_means, df, std_err[2]/(float)sqrt(2.0), 
      IMSLS_LSD,
      IMSLS_ALPHA, alpha,
      0);
   l_print_LSD(n_treatments, equal_means, treatment_means);

}

/*
* Function to display means comparison.
*/
void l_print_LSD(int n, int *equalMeans, float *means){
   float x=0.0;
   int i, j, k;
   int iSwitch;
   int *idx;

   idx = (int *) malloc(n * sizeof (int));

   for (k=0; k < n; k++) {
      idx[k]   =k+1;
   }

   /* Sort means in ascending order*/

   iSwitch=1;
   while (iSwitch != 0){
      iSwitch = 0;
      for (i = 0; i < n-1; i++){
         if (means[i] > means[i+1]){
            iSwitch = 1;
            x = means[i];
            means[i] = means[i+1];
            means[i+1] = x;
            j = idx[i];
            idx[i] = idx[i+1];
            idx[i+1] = j;
         }
      }
   }
   printf("[group] \t  Mean \t\tLSD Grouping \n");
   for (i=0; i < n; i++){
      printf("  [%d] \t\t%f", idx[i], means[i]);                
      for (j=1; j < i+1; j++){
         if(equalMeans[j-1] >= i+2-j){
            printf("\t  *");
         }else{ 
            if(equalMeans[j-1]>0) printf("\t");
         }
      }
      if (i < n-1 && equalMeans[i]>0) printf("\t  *");
      printf("\n");
   }
   free(idx);
   idx = NULL;
   return;
}
